@@ -1,4 +1,4 @@
-// This file Copyright © 2010-2022 Mnemosyne LLC.
+// This file Copyright © 2010-2023 Mnemosyne LLC.
 // It may be used under GPLv2 (SPDX: GPL-2.0-only), GPLv3 (SPDX: GPL-3.0-only),
 // or any future license endorsed by Mnemosyne LLC.
 // License text can be found in the licenses/ folder.
@@ -11,7 +11,6 @@
 
 #include <cstdint> // for size_t
 #include <cstdint> // for intX_t, uintX_t
-#include <ctime>
 #include <memory> // for std::unique_ptr
 #include <utility> // for std::pair
 #include <vector>
@@ -36,7 +35,7 @@ public:
     }
 
     // @return any error code from cacheTrim()
-    int writeBlock(tr_torrent_id_t tor, tr_block_index_t block, std::unique_ptr<std::vector<uint8_t>>& writeme);
+    int writeBlock(tr_torrent_id_t tor, tr_block_index_t block, std::unique_ptr<std::vector<uint8_t>> writeme);
 
     int readBlock(tr_torrent* torrent, tr_block_info::Location loc, uint32_t len, uint8_t* setme);
     int prefetchBlock(tr_torrent* torrent, tr_block_info::Location loc, uint32_t len);
@@ -50,36 +49,25 @@ private:
     {
         Key key;
         std::unique_ptr<std::vector<uint8_t>> buf;
-        time_t time_added = {};
     };
 
     using Blocks = std::vector<CacheBlock>;
     using CIter = Blocks::const_iterator;
 
-    struct CompareCacheBlockByKey
-    {
-        [[nodiscard]] constexpr bool operator()(Key const& key, CacheBlock const& block)
-        {
-            return key < block.key;
-        }
-        [[nodiscard]] constexpr bool operator()(CacheBlock const& block, Key const& key)
-        {
-            return block.key < key;
-        }
-    };
+    [[nodiscard]] static Key make_key(tr_torrent const* torrent, tr_block_info::Location loc) noexcept;
 
-    [[nodiscard]] static Key makeKey(tr_torrent const* torrent, tr_block_info::Location loc) noexcept;
+    [[nodiscard]] static std::pair<CIter, CIter> find_biggest_span(CIter begin, CIter end) noexcept;
 
-    [[nodiscard]] static std::pair<CIter, CIter> findContiguous(CIter const begin, CIter const end, CIter const iter) noexcept;
+    [[nodiscard]] static CIter find_span_end(CIter span_begin, CIter end) noexcept;
 
     // @return any error code from tr_ioWrite()
-    [[nodiscard]] int writeContiguous(CIter const begin, CIter const end) const;
+    [[nodiscard]] int write_contiguous(CIter begin, CIter end) const;
 
     // @return any error code from writeContiguous()
-    [[nodiscard]] int flushSpan(CIter const begin, CIter const end);
+    [[nodiscard]] int flush_span(CIter begin, CIter end);
 
     // @return any error code from writeContiguous()
-    [[nodiscard]] int flushOldest();
+    [[nodiscard]] int flush_biggest();
 
     // @return any error code from writeContiguous()
     [[nodiscard]] int cacheTrim();
@@ -98,4 +86,16 @@ private:
     mutable size_t disk_write_bytes_ = 0;
     mutable size_t cache_writes_ = 0;
     mutable size_t cache_write_bytes_ = 0;
+
+    static constexpr struct
+    {
+        [[nodiscard]] constexpr bool operator()(Key const& key, CacheBlock const& block)
+        {
+            return key < block.key;
+        }
+        [[nodiscard]] constexpr bool operator()(CacheBlock const& block, Key const& key)
+        {
+            return block.key < key;
+        }
+    } CompareCacheBlockByKey{};
 };
