@@ -329,11 +329,11 @@ struct MetainfoHandler final : public transmission::benc::BasicHandler<MaxBencDe
         }
         else if (pathIs(CommentKey) || pathIs(CommentUtf8Key))
         {
-            tm_.comment_ = tr_strv_replace_invalid(value);
+            tm_.comment_ = tr_strv_convert_utf8(value);
         }
         else if (pathIs(CreatedByKey) || pathIs(CreatedByUtf8Key))
         {
-            tm_.creator_ = tr_strv_replace_invalid(value);
+            tm_.creator_ = tr_strv_convert_utf8(value);
         }
         else if (
             pathIs(SourceKey) || pathIs(InfoKey, SourceKey) || //
@@ -344,7 +344,7 @@ struct MetainfoHandler final : public transmission::benc::BasicHandler<MaxBencDe
             // to have the same use as the 'source' key
             // http://wiki.bitcomet.com/inside_bitcomet
 
-            tm_.source_ = tr_strv_replace_invalid(value);
+            tm_.source_ = tr_strv_convert_utf8(value);
         }
         else if (pathIs(AnnounceKey))
         {
@@ -360,7 +360,7 @@ struct MetainfoHandler final : public transmission::benc::BasicHandler<MaxBencDe
         }
         else if (pathIs(InfoKey, NameKey) || pathIs(InfoKey, NameUtf8Key))
         {
-            tm_.name_ = tr_strv_replace_invalid(value);
+            tm_.setName(value);
         }
         else if (pathIs(InfoKey, PiecesKey))
         {
@@ -390,10 +390,10 @@ struct MetainfoHandler final : public transmission::benc::BasicHandler<MaxBencDe
         {
             tm_.addWebseed(value);
         }
-        else if (pathIs(MagnetInfoKey, DisplayNameKey))
+        else if (pathIs(MagnetInfoKey, DisplayNameKey) && std::empty(tm_.name()))
         {
             // compatibility with Transmission <= 3.0
-            tm_.name_ = tr_strv_replace_invalid(value);
+            tm_.setName(value);
         }
         else if (pathIs(MagnetInfoKey, InfoHashKey))
         {
@@ -408,6 +408,7 @@ struct MetainfoHandler final : public transmission::benc::BasicHandler<MaxBencDe
         else if (
             pathIs(ChecksumKey) || //
             pathIs(ErrCallbackKey) || //
+            pathIs(InfoKey, CrossSeedEntryKey) || //
             pathIs(InfoKey, Ed2kKey) || //
             pathIs(InfoKey, EntropyKey) || //
             pathIs(InfoKey, Md5sumKey) || //
@@ -421,6 +422,7 @@ struct MetainfoHandler final : public transmission::benc::BasicHandler<MaxBencDe
             pathIs(PublisherUrlKey) || //
             pathIs(PublisherUrlUtf8Key) || //
             pathIs(TitleKey) || //
+            pathIs(UidKey) || //
             pathStartsWith(AzureusPrivatePropertiesKey) || //
             pathStartsWith(AzureusPropertiesKey) || //
             pathStartsWith(InfoKey, CollectionsKey) || //
@@ -578,6 +580,7 @@ private:
     static constexpr std::string_view CreatedByKey = "created by"sv;
     static constexpr std::string_view CreatedByUtf8Key = "created by.utf-8"sv;
     static constexpr std::string_view CreationDateKey = "creation date"sv;
+    static constexpr std::string_view CrossSeedEntryKey = "cross_seed_entry"sv;
     static constexpr std::string_view DisplayNameKey = "display-name"sv;
     static constexpr std::string_view DurationKey = "duration"sv;
     static constexpr std::string_view Ed2kKey = "ed2k"sv;
@@ -621,6 +624,7 @@ private:
     static constexpr std::string_view Sha1Key = "sha1"sv;
     static constexpr std::string_view SourceKey = "source"sv;
     static constexpr std::string_view TitleKey = "title"sv;
+    static constexpr std::string_view UidKey = "uid"sv;
     static constexpr std::string_view UniqueKey = "unique"sv;
     static constexpr std::string_view UrlListKey = "url-list"sv;
     static constexpr std::string_view VcodecKey = "vcodec"sv;
