@@ -2,6 +2,8 @@
 // It may be used under the MIT (SPDX: MIT) license.
 // License text can be found in the licenses/ folder.
 
+#import "CocoaCompatibility.h"
+
 #import "GroupsPrefsController.h"
 #import "GroupsController.h"
 #import "ExpandedPathToPathTransformer.h"
@@ -41,6 +43,11 @@ typedef NS_ENUM(NSInteger, SegmentTag) {
 
     [self.fSelectedColorView addObserver:self forKeyPath:@"color" options:0 context:NULL];
 
+    if (@available(macOS 13.0, *))
+    {
+        self.fSelectedColorView.colorWellStyle = NSColorWellStyleMinimal;
+    }
+
     [self updateSelectedGroup];
 }
 
