@@ -1,4 +1,4 @@
-// This file Copyright © 2012-2022 Mnemosyne LLC.
+// This file Copyright © 2012-2023 Mnemosyne LLC.
 // It may be used under GPLv2 (SPDX: GPL-2.0-only), GPLv3 (SPDX: GPL-3.0-only),
 // or any future license endorsed by Mnemosyne LLC.
 // License text can be found in the licenses/ folder.
@@ -12,7 +12,7 @@ class CustomVariantType
 public:
     enum
     {
-        TrackerStatsList = QVariant::UserType,
+        TrackerStatsList = QMetaType::User,
         PeerList,
         FileList,
         FilterModeType,
