@@ -23,13 +23,7 @@ import {
   TorrentRendererCompact,
   TorrentRendererFull,
 } from './torrent-row.js';
-import {
-  debounce,
-  deepEqual,
-  setEnabled,
-  setTextContent,
-  movePopup,
-} from './utils.js';
+import { debounce, deepEqual, setEnabled, setTextContent } from './utils.js';
 
 export class Transmission extends EventTarget {
   constructor(action_manager, notifications, prefs) {
@@ -148,15 +142,6 @@ export class Transmission extends EventTarget {
                 this.action_manager
               )
             );
-            const btnbox = document
-              .querySelector('#toolbar-overflow')
-              .getBoundingClientRect();
-            movePopup(
-              this.popup.root,
-              btnbox.left + btnbox.width,
-              btnbox.top + btnbox.height,
-              document.body
-            );
           }
           break;
         case 'show-preferences-dialog':
@@ -236,12 +221,19 @@ export class Transmission extends EventTarget {
 
       const popup = new ContextMenu(this.action_manager);
       this.setCurrentPopup(popup);
-      movePopup(
-        popup.root,
+
+      const boundingElement = document.querySelector('#torrent-container');
+      const bounds = boundingElement.getBoundingClientRect();
+      const x = Math.min(
         event_.x,
-        event_.y,
-        document.querySelector('#torrent-container')
+        bounds.x + bounds.width - popup.root.clientWidth
       );
+      const y = Math.min(
+        event_.y,
+        bounds.y + bounds.height - popup.root.clientHeight
+      );
+      popup.root.style.left = `${x > 0 ? x : 0}px`;
+      popup.root.style.top = `${y > 0 ? y : 0}px`;
       event_.preventDefault();
     });
 
